from conf import*   # fiber specific modules ( symbol, token ) and objects ( prompt, pprint, unparse ...)
import EasyExtend   # EasyExtend package
import PyGrammar    # contains fiber specific DFA tables used by the parser
from EasyExtend.csttools import*   # tools used to manipulate CSTs
from EasyExtend.eetransformer import Transformer, transform

import EasyExtend.fibers.macro.fiber as macro_fiber

def operator_concat(*args):
    '''
    Defines behaviour of '++'.
    Use __concat__ protocol. For types in list, tuple and str use the __add__ operator instead.
    '''
    a,b = args[:2]
    if a in (basestring, list, tuple):
       res = a+b
    else:
        if hasattr(a,"__concat__"):
            res = a.__concat__(b)
        elif hasattr(b,"__rconcat__"):
            res = b.__rconcat__(a)
        else:
            raise TypeError(":unsupported operand type(s) for ++: '%s' and '%s'"%(type(a), type(b)))
    if args[2:]:
        return operator_concat(*(res,)+args[2:])
    return res

class Hex(object):
    def __init__(self, *args):
        self.bytes = []
        for arg in args:
            if isinstance(arg, (int,long)):
                self.bytes.append(self.create_from_int(arg).bytes)
            elif isinstance(arg, Hex):
                self.bytes.append(arg.bytes)
            else:
                raise TypeError,"cannot convert to Hex object: '%s'"%arg
        self.bytes = sum(self.bytes,[])

    @classmethod
    def create_from_int(cls, i):
        assert isinstance(i,(int,long)), "type of i is %s"%type(i)
        h = Hex()
        h.bytes = []
        v = i
        if v == 0:
            h.bytes = [0]
        else:
            while v:
                v,r = divmod(v,0x100)
                h.bytes.insert(0,r)
        return h

    def __len__(self):
        return len(self.bytes)

    def __concat__(self, other):
        if isinstance(other, Hex):
            h = Hex()
            h.bytes = self.bytes+other.bytes
            return h
        else:
            h = Hex()
            h.bytes = self.bytes+Hex(other).bytes
            return h

    def __rconcat__(self, other):
        return Hex(other).__concat__(self)

    def __repr__(self):
        return " ".join(["0x%02X"%item for item in self.bytes])


class FiberTransformer(Transformer):
    '''
    Defines fiber specific transformations
    '''

    @transform
    def numbers(self, node):
        '''
        numbers: NUMBER+
        '''
        _numbers = find_all(node, token.NUMBER)
        if len(_numbers)==1:
            if not _numbers[0][1].startswith("0x"):
                left_shift(node)
                return atom(_numbers[0])
            else:
                return CST_CallFunc("Hex",[_numbers[0]])
        else:
            return CST_CallFunc("Hex",_numbers)

    @transform
    def comparison(self, node):
        if find_node(node, token.PLUSPLUS, level = 1):
            _expr = find_all(node, symbol.expr, level = 1)
            mc = macro_fiber.macro("operator_concat(*<expr>)")
            return mc.expand({"expr": _expr}, returns = symbol.comparison)


__publish__ = ["Hex", "operator_concat"]

if __name__ == '__main__':
    import fiber
    (options, args) = opt.parse_args()
    fiber.options   = options.__dict__
    if args:
        py_module = args[-1]
        EasyExtend.run_module(py_module, fiber)
    else:
        console = EasyExtend.create_console(fiber.fiber_name, fiber)
        console.interact()

