#! /usr/bin/env python
"""
Test of csttools.py module
"""

from EasyExtend.csttools import*
from EasyExtend.cst2source import*
import parser
import symbol
import unittest
from test import test_support

SHOW = True


def compare_expr(expr, expr_id, tree, show = False):
    if show:
        print
        print "P:",find_node(parser.suite(expr).tolist(),expr_id)
        print
        print "T:",tree
    return find_node(parser.suite(expr).tolist(),expr_id) == tree

class AtomTestCase(unittest.TestCase):
    "atom: '(' [testlist_gexp] ')' | '[' [listmaker] ']' | '{' [dictmaker] '}' | '`' testlist1 '`' | NAME | NUMBER | STRING+"

    def test_String(self):
        x = atom(String("any_name"), String("another string"))
        self.assertEqual(len(x), 3)

    def test_Name(self):
        self.assertEqual(atom(Name("any_name"),Name("any_name")), atom(Name("any_name")))

    def test_Number(self):
        self.assertEqual(atom(Number(9),Number(7)), atom(Number(9)))


    def test_dictmaker(self):
        # dictmaker: test ':' test (',' test ':' test)* [',']
        n = any_test(Number(9))
        x = any_test(Name("x"))
        y = any_test(Name("y"))
        self.assertEqual(compare_expr("{9:x}",symbol.dictmaker, dictmaker(n,x)), True)
        self.assertRaises(AssertionError,dictmaker,n,x,x)
        self.assertEqual(compare_expr("{9:x,}",symbol.dictmaker, dictmaker(n,x,",")), True)
        self.assertEqual(compare_expr("{9:x,x:y}",symbol.dictmaker, dictmaker(n,x,x,y)), True)
        self.assertEqual(compare_expr("{9:x}", symbol.atom, atom("{",dictmaker(n,x),"}")),True)

    def test_listmaker(self):
        # listmaker: test ( list_for | (',' test)* [','] )
        n = any_test(Number(9))
        x = any_test(Name("x"))
        y = any_test(String("y"))
        self.assertEqual(compare_expr("[9]",symbol.listmaker, listmaker(n)), True)
        self.assertEqual(compare_expr("[9,x]",symbol.listmaker, listmaker(n,x)), True)
        self.assertEqual(compare_expr('[9,x,"y"]',symbol.listmaker, listmaker(n,x,y)), True)
        self.assertEqual(compare_expr("[9,x,]",symbol.listmaker, listmaker(n,x,",")), True)

    def test_exprlist(self):
        # exprlist: expr (',' expr)* [',']
        n = any_expr(Number(9))
        x = any_expr(Name("x"))
        self.assertNotEqual(exprlist(n,n),exprlist(n))
        self.assertNotEqual(exprlist(n,x,","),exprlist(n,x))

    def test_list_for(self):
        # list_for: 'for' exprlist 'in' testlist_safe [list_iter]
        # list_iter: list_for | list_if
        # list_if: 'if' test [list_iter]
        # testlist_safe: test [(',' test)+ [',']]
        n = any_test(Number(9))
        x = any_test(Name("x"))
        y = any_test(String("y"))
        en = find_node(n,symbol.expr)
        ex = find_node(x,symbol.expr)

        list_if(n)
        list_iter(list_if(n))
        list_if(n,list_iter(list_if(n)))
        list_for(exprlist(en),testlist_safe(x))
        list_for(exprlist(en),testlist_safe(x,n))

    def test_listcomp(self):
        x  = any_test(Name("x"))
        y  = any_test(Name("y"))
        yl = exprlist(any_expr(Name("y")))
        lcomp = listmaker(y,list_for(yl,testlist_safe(x)))
        self.assertEqual(compare_expr("[y for y in x]",symbol.listmaker, lcomp), True)
        self.assertEqual(compare_expr("[y for y in x]", symbol.atom, atom("[",lcomp,"]")),True)

    def test_gen_for(self):
        # gen_for: 'for' exprlist 'in' test [gen_iter]
        x  = any_test(Name("x"))
        y  = any_expr(Name("y"))
        z  = any_expr(Name("z"))
        yl = exprlist(y,z)
        gf = gen_for(yl,x,gen_iter(gen_for(yl,x)))
        self.assertEqual(compare_expr("(y for y,z in x)",symbol.gen_for, gen_for(yl,x)),True)
        self.assertEqual(compare_expr("(y for y,z in x for y,z in x)",symbol.gen_for, gf),True)



    def test_testlist_gexp(self):
        # test ( gen_for | (',' test)* [','] )
        x  = any_test(Name("x"))
        y  = any_test(Name("y"))
        a  = any_expr(Name("a"))
        b  = any_expr(Name("b"))
        ab = exprlist(a,b)

        self.assertEqual(compare_expr("(x, y)", symbol.atom, atom('(',testlist_gexp(x,y),')')),True)
        self.assertEqual(compare_expr("(x, y,)", symbol.atom, atom('(',testlist_gexp(x,y,","),')')),True)
        self.assertEqual(compare_expr("(x)", symbol.atom, atom('(',testlist_gexp(x),')')),True)
        self.assertEqual(compare_expr("(x)", symbol.atom, atom('(',testlist_gexp(x),')')),True)
        self.assertEqual(compare_expr("(x for a,b in y)", symbol.atom, atom('(',testlist_gexp(x,gen_for(ab,y)),')')),True)


class TrailerTestCase(unittest.TestCase):
    "trailer: '(' [arglist] ')' | '[' subscriptlist ']' | '.' NAME"

    def test_dot_name(self):
        self.assertEqual(compare_expr("x.any_name",symbol.trailer,trailer(".",Name("any_name"))),True)

    def test_subscript(self):
        # subscript: '.' '.' '.' | [test] ':' [test] [sliceop] | test
        sl   = sliceop()
        sl_t = sliceop(any_test(Number(1)))
        subscript("...")
        self.assertEqual(compare_expr("l[::]",symbol.subscript, subscript(":",sl)), True)
        self.assertEqual(compare_expr("l[::1]",symbol.subscript, subscript(":",sl_t)), True)
        self.assertEqual(compare_expr("l[1]",symbol.subscript, subscript(any_test(Number(1)))), True)
        self.assertEqual(compare_expr("l[1:]",symbol.subscript, subscript(any_test(Number(1)),':')), True)
        self.assertEqual(compare_expr("l[1::]",symbol.subscript, subscript(any_test(Number(1)),':',sl)), True)
        self.assertEqual(compare_expr("l[1:3]",symbol.subscript, subscript(any_test(Number(1)),':',any_test(Number(3)))), True)

    def test_subscriptlist(self):
        sl   = sliceop()
        sl_t = sliceop(any_test(Number(1)))
        ss0  = subscript(":",sl)
        ss1  = subscript(":",sl_t)
        ss2  = subscript(any_test(Number(3)))
        self.assertEqual(compare_expr("l[3]",symbol.subscriptlist, subscriptlist(ss2)), True)
        self.assertEqual(compare_expr("l[::1]",symbol.subscriptlist, subscriptlist(ss1)), True)
        self.assertEqual(compare_expr("l[::]",symbol.subscriptlist, subscriptlist(ss0)), True)
        self.assertEqual(compare_expr("l[::,::]",symbol.subscriptlist, subscriptlist(ss0,ss0)), True)
        self.assertEqual(compare_expr("l[::,::1]",symbol.subscriptlist, subscriptlist(ss0,ss1)), True)
        self.assertEqual(compare_expr("l[::,::1,]",symbol.subscriptlist, subscriptlist(ss0,ss1,',')), True)

    def test_argument(self):
        # argument: [test '='] test [gen_for]
        a  = any_test(Name("a"))
        b  = any_test(Name("b"))
        y  = any_expr(Name("y"))
        z  = any_expr(Name("z"))
        yl = exprlist(y,z)
        self.assertEqual(compare_expr("f(a)",symbol.argument,argument(a)),True)
        self.assertEqual(compare_expr("f(a=b)",symbol.argument,argument(a,'=',b)),True)
        self.assertEqual(compare_expr("f(b for y,z in a)",symbol.argument,argument(b,gen_for(yl,a))),True)
        #self.assertEqual(compare_expr("f(a = b for y,z in a)",symbol.argument,argument(a,'=',b,gen_for(yl,a))),True)


    def test_arglist(self):
        # arglist: (argument ',')* (argument [',']| '*' test [',' '**' test] | '**' test)
        a  = any_test(Name("a"))
        b  = any_test(Name("b"))
        y  = any_expr(Name("y"))
        z  = any_expr(Name("z"))
        yl = exprlist(y,z)
        a1 = argument(a)
        b1 = argument(b)
        a2 = argument(a,'=',b)
        a3 = argument(b,gen_for(yl,a))
        a4 = argument(a,'=',b,gen_for(yl,a))
        self.assertEqual(compare_expr("f(a)",symbol.arglist,arglist(a1)),True)
        self.assertEqual(compare_expr("f(a,b)",symbol.arglist,arglist(a1,b1)),True)
        self.assertEqual(compare_expr("f(a,b,)",symbol.arglist,arglist(a1,b1,",")),True)
        self.assertEqual(compare_expr("f(*a)",symbol.arglist,arglist('*',a)),True)
        self.assertEqual(compare_expr("f(*a,**b)",symbol.arglist,arglist('*',a,'**',b)),True)
        self.assertEqual(compare_expr("f(**b)",symbol.arglist,arglist('**',b)),True)
        self.assertEqual(compare_expr("f(a=b,b=a)",symbol.arglist,arglist(argument(a,'=',b),argument(b,'=',a))),True)
        #self.assertEqual(compare_expr("f(a = b for y,z in a)",symbol.arglist,arglist(a4)),True)


    def test_trailer(self):
        # trailer : '(' [arglist] ')' | '[' subscriptlist ']' | '.' NAME
        ssl = subscriptlist(subscript(any_test(Number(3))))
        self.assertEqual(compare_expr("f()",symbol.trailer, trailer("(",")")),True)
        self.assertEqual(compare_expr("f[3]",symbol.trailer, trailer("[",ssl,"]")),True)
        a  = any_test(Name("a"))
        b  = any_test(Name("b"))
        a1 = argument(a)
        b1 = argument(b)
        self.assertEqual(compare_expr("f(a,b)",symbol.trailer, trailer("(",arglist(a1,b1),")")),True)
        self.assertEqual(compare_expr("f.a",symbol.trailer, trailer(".",Name("a"))),True)


class ExprTestCase(unittest.TestCase):
    def test_power_factor(self):
        # power: atom trailer* ['**' factor]
        x = atom(Name("x"))
        X = factor(power(x))
        t = trailer("(",")")
        self.assertEqual(compare_expr("x()",symbol.power,power(x,t)),True)
        self.assertEqual(compare_expr("x()()",symbol.power,power(x,t,t)),True)
        # factor: ('+'|'-'|'~') factor | power
        factor(power(x))
        factor('+',X)
        factor('-',X)
        factor('~',factor('~',X))
        factor('+',factor('-',X))
        self.assertEqual(compare_expr("x**x",symbol.power,power(x,'**',X)),True)
        self.assertEqual(compare_expr("x()()**x",symbol.power,power(x,t,t,'**',X)),True)
        self.assertEqual(compare_expr("x**-x",symbol.power,power(x,'**',factor('-',X))),True)

    def test_term(self):
        # term: factor (('*'|'/'|'%'|'//') factor)*
        x = atom(Name("x"))
        y = atom(Name("y"))
        X = factor(power(x))
        Y = factor(power(y))
        self.assertEqual(compare_expr("x*y",symbol.term,term(X,'*',Y)),True)
        self.assertEqual(compare_expr("x*y%x",symbol.term,term(X,'*',Y,'%',X)),True)
        self.assertEqual(compare_expr("x/y",symbol.term,term(X,'/',Y)),True)
        self.assertEqual(compare_expr("x//y",symbol.term,term(X,'//',Y)),True)
        self.assertEqual(compare_expr("x//y*x",symbol.term,term(X,'//',Y,'*',X)),True)

    def test_arith_expr(self):
        x = atom(Name("x"))
        y = atom(Name("y"))
        X = factor(power(x))
        Y = factor(power(y))
        TX=term(X)
        TY=term(Y)
        TXY = term(X,'*',Y)
        self.assertEqual(compare_expr("x",symbol.arith_expr,arith_expr(TX)),True)
        self.assertEqual(compare_expr("x+y",symbol.arith_expr,arith_expr(TX,'+',TY)),True)
        self.assertEqual(compare_expr("x-y+x",symbol.arith_expr,arith_expr(TX,'-',TY,'+',TX)),True)

    def test_shift_expr(self):
        x = atom(Name("x"))
        y = atom(Name("y"))
        X = term(factor(power(x)))
        Y = term(factor(power(y)))
        AX= arith_expr(X)
        AY= arith_expr(Y)
        AXY = arith_expr(X,'+',Y)
        self.assertEqual(compare_expr("x",symbol.shift_expr, shift_expr(AX)),True)
        self.assertEqual(compare_expr("x<<y",symbol.shift_expr, shift_expr(AX,'<<',AY)),True)
        self.assertEqual(compare_expr("x>>y>>x+y",symbol.shift_expr, shift_expr(AX,'>>',AY,'>>',AXY)),True)

    def test_and_expr(self):
        x = atom(Name("x"))
        y = atom(Name("y"))
        X = arith_expr(term(factor(power(x))))
        Y = arith_expr(term(factor(power(y))))
        SX= shift_expr(X)
        SY= shift_expr(Y)
        SXY = shift_expr(X,'<<',Y)
        self.assertEqual(compare_expr("x",symbol.and_expr, and_expr(SX)),True)
        self.assertEqual(compare_expr("x&y",symbol.and_expr, and_expr(SX,SY)),True)
        self.assertEqual(compare_expr("x&y&x<<y",symbol.and_expr, and_expr(SX,SY,SXY)),True)

    def test_xor_expr(self):
        x = atom(Name("x"))
        y = atom(Name("y"))
        X = shift_expr(arith_expr(term(factor(power(x)))))
        Y = shift_expr(arith_expr(term(factor(power(y)))))
        AX= and_expr(X)
        AY= and_expr(Y)
        AXY = and_expr(X,Y)
        self.assertEqual(compare_expr("x",symbol.xor_expr, xor_expr(AX)),True)
        self.assertEqual(compare_expr("x^y",symbol.xor_expr, xor_expr(AX,AY)),True)
        self.assertEqual(compare_expr("x^y^x&y",symbol.xor_expr, xor_expr(AX,AY,AXY)),True)

    def test_expr(self):
        x = atom(Name("x"))
        y = atom(Name("y"))
        X = and_expr(shift_expr(arith_expr(term(factor(power(x))))))
        Y = and_expr(shift_expr(arith_expr(term(factor(power(y))))))
        XX= xor_expr(X)
        XY= xor_expr(Y)
        XXY = xor_expr(X,Y)
        self.assertEqual(compare_expr("x",symbol.expr, expr(XX)),True)
        self.assertEqual(compare_expr("x|y",symbol.expr, expr(XX,XY)),True)
        self.assertEqual(compare_expr("x|y|x^y",symbol.expr, expr(XX,XY,XXY)),True)


class TestTestCase(unittest.TestCase):
    "test: and_test ('or' and_test)* | lambdef "

    def test_comparison(self):
        comp_op("<")
        comp_op(">")
        comp_op("==")
        comp_op(">=")
        comp_op("<=")
        comp_op("!=")
        comp_op("in")
        comp_op("not in")
        comp_op("is")
        comp_op("is not")
        x = any_expr(Name("x"))
        y = any_expr(Name("y"))
        self.assertEqual(compare_expr("x",symbol.comparison, comparison(x)),True)
        self.assertEqual(compare_expr("x<y==y",symbol.comparison, comparison(x,"<",y,"==",y)),True)

    def test_not_test(self):
        x = comparison(any_expr(Name("x")))
        y = comparison(any_expr(Name("y")))
        self.assertEqual(compare_expr("x",symbol.not_test, not_test(x)),True)
        self.assertEqual(compare_expr("not x",symbol.not_test, not_test('not',not_test(x))),True)
        self.assertEqual(compare_expr("not not x",symbol.not_test, not_test('not',not_test('not',not_test(x)))),True)

    def test_and_test(self):
        x = not_test(comparison(any_expr(Name("x"))))
        y = not_test(comparison(any_expr(Name("y"))))
        self.assertEqual(compare_expr("x",symbol.and_test, and_test(x)),True)


class ImportTestCase(unittest.TestCase):
    def test_import_name(self):
        name = "module"
        import_module = any_stmt(import_stmt(import_name(dotted_as_names(dotted_as_name(dotted_name(name))))))
        self.assertEqual(compare_expr("import module",symbol.stmt, import_module),True)

    def test_import_as_name(self):
        mod_name   = "module"
        as_module  = "other_module"
        pack_name  = "package"
        dotted_import_as = dotted_as_names(dotted_as_name(dotted_name(pack_name, mod_name),'as',as_module))
        import_module = any_stmt(import_stmt(import_name(dotted_import_as)))
        self.assertEqual(compare_expr("import package.module as other_module",symbol.stmt, import_module,show=True),True)

    def test_import_from(self):
        mod_name  = "module"
        pack_name = "package"
        as_module = "module2"
        import_module = any_stmt(import_stmt(import_from(dotted_name(mod_name),'*')))
        self.assertEqual(compare_expr("from module import*",symbol.stmt, import_module),True)
        import_pack_module = any_stmt(import_stmt(import_from(dotted_name(pack_name,mod_name),'*')))
        self.assertEqual(compare_expr("from package.module import*",symbol.stmt, import_pack_module),True)
        imp_as = import_as_names(import_as_name(mod_name,as_module))
        import_pack_module_as = any_stmt(import_stmt(import_from(dotted_name(pack_name),imp_as)))
        self.assertEqual(compare_expr("from package import module as module2",symbol.stmt, import_pack_module_as),True)

def test_main():
    test_support.run_unittest(AtomTestCase)
    test_support.run_unittest(TrailerTestCase)
    test_support.run_unittest(ExprTestCase)
    test_support.run_unittest(TestTestCase)
    test_support.run_unittest(ImportTestCase)


if __name__ == "__main__":
    test_main()





